local assets = {
    Asset( "SOUND", "sound/markimod.fsb" ),
	Asset( "ANIM", "anim/blacksmithsword.zip"),
	Asset( "ANIM", "anim/swap_blacksmithsword.zip"),
}

local prefabs = {
	"bleed",
	"bubbles",
	"blacksmithportal"
}

local function getrandomposition(inst)
	local ground = TheWorld
	local centers = {}
	for i,node in ipairs(ground.topology.nodes) do
		local tile = TheWorld.Map:GetTileAtPoint(node.x, 0, node.y)
		if tile and tile ~= GROUND.IMPASSABLE then
			table.insert(centers, {x = node.x, z = node.y})
		end
	end
	if #centers > 0 then
		local pos = centers[math.random(#centers)]
		return Point(pos.x, 0, pos.z)
	else
		return inst.components.inventoryitem.owner:GetPosition()
	end
end

local function onattack(inst, attacker, target)
	inst.SoundEmitter:PlaySound("markimod/markimod/fx/swordslash")
	if MarkMod.BlacksmithSwordPower then
		if not target.SoundEmitter then
			target.entity:AddSoundEmitter()
		end
		local Special = math.random(1,100)
		if Special <= 20 then
			local ChoosePower = math.random(1,2)
			if ChoosePower == 1 then
				if target.components.burnable and not target.components.burnable:IsBurning() then
					if target.components.freezable and target.components.freezable:IsFrozen() then           
						target.components.freezable:Unfreeze()            
					else            
						target.components.burnable:Ignite(true)
					end   
				end
			elseif ChoosePower == 2 then
				if target.components.freezable then
					target.components.freezable:AddColdness(10)
					target.components.freezable:SpawnShatterFX()
				end
				if target.components.burnable and target.components.burnable:IsBurning() then
					target.components.burnable:Extinguish()
				end
			end
		elseif Special >= 99 then
			if not inst:HasTag("blacksmithimmune") then
				if target.components.locomotor then
					target.components.locomotor:StopMoving()
				end
				SpawnPrefab("blacksmithportal").Transform:SetPosition(target:GetPosition():Get())
				target.SizeReduction = 1
				if target.Reduce == nil then
					target.components.health:SetInvincible(true)
					target.Reduce = target:DoPeriodicTask(0.1, function()
						target.SizeReduction = target.SizeReduction - 0.1
						target.Transform:SetScale(target.SizeReduction,target.SizeReduction,target.SizeReduction)
						if target.SizeReduction <= 0.1 then
							target.Reduce:Cancel()
							target.Reduce = nil
							local t_loc = getrandomposition(inst)
							target.Transform:SetPosition(t_loc.x, t_loc.y, t_loc.z)
							target.Transform:SetScale(1,1,1)
							target.components.health:SetInvincible(false)
						end
					end)
				end
			end
		end
	end
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_blacksmithsword", "swap_ruins_bat")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function onfinished(inst)
	local owner = inst.components.inventoryitem.owner
	local Platinum = SpawnPrefab("blacksmithplatinum")
	owner.components.inventory:GiveItem(Platinum)
	SpawnPrefab("brokentool").Transform:SetPosition(owner:GetPosition():Get())
	owner.AnimState:PlayAnimation("hit")
	owner.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
	inst:Remove()
end

local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddNetwork()
	MakeInventoryPhysics(inst)
   
	inst:AddTag("BLACKSMITHSWORD")
	inst:AddTag("sharp")
    
    inst.AnimState:SetBank("ruins_bat")
    inst.AnimState:SetBuild("blacksmithsword")
    inst.AnimState:PlayAnimation("idle")
	
	inst.entity:SetPristine()
	if not TheWorld.ismastersim then
		return inst
	end
	MakeHauntableLaunch(inst)
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(30)
    inst.components.weapon:SetOnAttack(onattack)
	
    inst:AddComponent("inspectable")
	
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/blacksmithsword.xml"
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip )
	
	inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(20)
    inst.components.finiteuses:SetUses(200)
    inst.components.finiteuses:SetOnFinished(onfinished)
	
    return inst
end

return Prefab( "common/inventory/blacksmithsword", fn, assets) 