local MakePlayerCharacter = require "prefabs/player_common"
local C = STRINGS.CHARACTERS

local assets = {
    Asset( "SOUND", "sound/markimod.fsb" ),
    Asset( "ANIM", "anim/markiplier.zip" ),
	Asset( "ANIM", "anim/markdark.zip" ),
	Asset( "ANIM", "anim/markmaniac.zip" ),
	Asset( "ANIM", "anim/ghost_markiplier_build.zip" )
}

local prefabs = {
	"blacksmithplatinum",
	"tinytimdevice",
	"warfstache",
	"fedora",
	"blacksmithnote"
}

local start_inv = {
	"tinytimdevice",
	"warfstache",
	"fedora",
	--"blacksmithnote",
	"blacksmithplatinum",
	"blacksmithplatinum",
}

local MANIAC_DIET = {FOODTYPE.MEAT}
local MANIAC_LMB_ACTIONS = { "CHOP", "MINE", "DIG" }
local MANIAC_RMB_ACTIONS = { "HAMMER" }
local MANIAC_ACTION_TAGS = { "CHOP_workable", "MINE_workable", "DIG_workable"}
local MANIAC_TARGET_EXCLUDE_TAGS = { "FX", "NOCLICK", "DECOR", "INLIMBO", "catchable" }

local function maniacactionstring(inst, action)
    return (action.action == ACTIONS.EAT and STRINGS.ACTIONS.EAT) or STRINGS.ACTIONS.BONK
end

local function GetManiacAction(target)
    for i, v in ipairs(MANIAC_LMB_ACTIONS) do
        if target:HasTag(v.."_workable") then
            return ACTIONS[v]
        end
    end
end

local function ManiacActionButton(inst, force_target)
    if not inst.components.playercontroller:IsDoingOrWorking() then
        if force_target == nil then
            local x, y, z = inst.Transform:GetWorldPosition()
            local ents = TheSim:FindEntities(x, y, z, inst.components.playercontroller.directwalking and 3 or 6, nil, MANIAC_TARGET_EXCLUDE_TAGS, MANIAC_ACTION_TAGS)
            for i, v in ipairs(ents) do
                if v ~= inst and v.entity:IsVisible() and CanEntitySeeTarget(inst, v) then
                    local action = GetManiacAction(v)
                    if action ~= nil then
                        return BufferedAction(inst, v, action)
                    end
                end
            end
        elseif inst:GetDistanceSqToInst(force_target) <= (inst.components.playercontroller.directwalking and 9 or 36) then
            local action = GetManiacAction(force_target)
            if action ~= nil then
                return BufferedAction(inst, force_target, action)
            end
        end
    end
end

local function LeftClickPicker(inst, target)
    if target ~= nil and target ~= inst then
        if inst.replica.combat:CanTarget(target) then
            return (not target:HasTag("player") or inst.components.playercontroller:IsControlPressed(CONTROL_FORCE_ATTACK))
                and inst.components.playeractionpicker:SortActionList({ ACTIONS.ATTACK }, target, nil)
                or nil
        end
        for i, v in ipairs(MANIAC_LMB_ACTIONS) do
            if target:HasTag(v.."_workable") then
                return inst.components.playeractionpicker:SortActionList({ ACTIONS[v] }, target, nil)
            end
        end
    end
end

local function RightClickPicker(inst, target)
    if target ~= nil and target ~= inst then
        for i, v in ipairs(MANIAC_DIET) do
            if target:HasTag("edible_"..v) then
                return inst.components.playeractionpicker:SortActionList({ ACTIONS.EAT }, target, nil)
            end
        end
        for i, v in ipairs(MANIAC_RMB_ACTIONS) do
            if target:HasTag(v.."_workable") then
                return inst.components.playeractionpicker:SortActionList({ ACTIONS[v] }, target, nil)
            end
        end
    end
end

local function ManiacMode(inst, enable)
	if enable == true then
		if inst.HUD ~= nil then
			inst.HUD.controls.crafttabs:Hide()
			inst.HUD.controls.inv:Hide()
			inst.HUD.controls.status.stomach:Hide()
			inst.HUD.controls.status.brain:Hide()
			local HCP = inst.HUD.controls.status.heart:GetPosition()
			local HSP = inst.HUD.controls.status.heart
			HSP:SetPosition(HCP.x - 40, HCP.y, 0)
		end
		
		TheWorld:PushEvent("enabledynamicmusic", false)
		if not inst.SoundEmitter:PlayingSound("markmetal") then
			inst.SoundEmitter:PlaySound("markimod/markimod/music/shia_mark_metal-mSb/loop", "markmetal")
		end
		
		inst.ActionStringOverride = maniacactionstring
		inst.CanExamine = false
		
		if TheWorld.ismastersim then
			if inst.components.worker == nil then
				inst:AddComponent("worker")
				inst.components.worker:SetAction(ACTIONS.CHOP, 1.2)
				inst.components.worker:SetAction(ACTIONS.MINE, 1)
				inst.components.worker:SetAction(ACTIONS.DIG, .50)
				inst.components.worker:SetAction(ACTIONS.HAMMER, .50)
			end

			if inst.components.playercontroller ~= nil then
				inst.components.playercontroller.actionbuttonoverride = ManiacActionButton
			end
			if inst.components.playeractionpicker ~= nil then
				inst.components.playeractionpicker.leftclickoverride = LeftClickPicker
				inst.components.playeractionpicker.rightclickoverride = RightClickPicker
			end
			
			inst.components.playervision:ForceNightVision(true)
		end
	else
		if inst.HUD ~= nil then
			inst.HUD.controls.crafttabs:Show()
			inst.HUD.controls.inv:Show()
			inst.HUD.controls.status.stomach:Show()
			inst.HUD.controls.status.brain:Show()
			local HCP = inst.HUD.controls.status.heart:GetPosition()
			local HSP = inst.HUD.controls.status.heart
			HSP:SetPosition(HCP.x + 40, HCP.y, 0)
		end
		
		TheWorld:PushEvent("enabledynamicmusic", true)
		inst.SoundEmitter:KillSound("markmetal")
		
		inst.ActionStringOverride = nil
		inst.CanExamine = nil
		
		if TheWorld.ismastersim then
			if inst.components.worker ~= nil then inst:RemoveComponent("worker") end
			
			if inst.components.playercontroller ~= nil then
				inst.components.playercontroller.actionbuttonoverride = nil
			end
			if inst.components.playeractionpicker ~= nil then
				inst.components.playeractionpicker.leftclickoverride = nil
				inst.components.playeractionpicker.rightclickoverride = nil
			end
			
			inst.components.playervision:ForceNightVision(false)
		end
	end
end

local function ToManiac(inst)
	inst:AddTag("Maniac")
	inst:AddTag("monster")
	inst.AnimState:SetBuild("markmaniac")
	inst.AnimState:SetBank("markmaniac")
	ManiacMode(inst, true)
	if TheWorld.ismastersim then
		if inst.sg ~= nil then inst:SetStateGraph("SGmarkmaniac") end
		inst.components.hunger:Pause()
		inst.components.sanity.ignore = true
		inst.components.eater:SetCanEatHorrible()
		inst.components.eater:SetCanEatRaw()
		inst.components.eater.strongstomach = true
		
		inst.components.pinnable.canbepinned = false
		inst.components.temperature.inherentinsulation = TUNING.INSULATION_LARGE
		inst.components.temperature.inherentsummerinsulation = TUNING.INSULATION_LARGE
		inst.components.moisture:SetInherentWaterproofness(TUNING.WATERPROOFNESS_LARGE)
		inst.components.catcher:SetEnabled(false)
	end
end

local function FromManiac(inst)
	inst:RemoveTag("Maniac")
	inst:RemoveTag("monster")
	inst.AnimState:SetBuild("markiplier")
	inst.AnimState:SetBank("wilson")
	ManiacMode(inst, false)
	if TheWorld.ismastersim then
		inst:SetStateGraph("SGwilson")
		inst.components.hunger:Resume()
		inst.components.sanity.ignore = false
		inst.components.eater.strongstomach = nil
		
		inst.components.pinnable.canbepinned = true
		inst.components.temperature.inherentinsulation = 0
		inst.components.temperature.inherentsummerinsulation = 0
		inst.components.moisture:SetInherentWaterproofness(0)
		inst.components.catcher:SetEnabled(true)
	end
end

local FORMS = {Markiplier = 1, Wilford = 2, Fedoraplier = 3, Darkiplier = 4, Maniac = 5}
local STATES = {
	[FORMS.Markiplier] = {
		form = "Markiplier", 		regen = {0,0}, 								absorb = 0,
		hunger = 1.05, 				dapper = {"MED", -0.2}, 					negaura = 1.2,
		nightdrain = 0.9, 			dmg = 1.1, 									scale = 0.95,
		icon = "markiplier", 		sound = "markiplier", 						--name = "Markiplier",
		speech = "markiplier_base", diet = {FOODGROUP.OMNI}, 					wlkspd = 1,
		runspd = 1, 				talkercolor = {255,255,255}, 				
		
		specialfunction = function(inst, from)
			if from == "Darkiplier" then
				inst:RemoveTag("monster")
				inst:RemoveTag("blacksmithimmune")
				inst.AnimState:SetBuild("markiplier")
			elseif from == "Maniac" then
				FromManiac(inst)
			end
		end
	},
	[FORMS.Wilford] = {
		form = "Wilford", 			regen = {0.1,1}, 							absorb = 0.2,
		hunger = 7, 				dapper = {"MED", 0.4}, 						negaura = 0.7,
		nightdrain = 0.7, 			dmg = 1.8, 									scale = 1,
		icon = "markipliertwo", 	sound = "wilford", 							--name = "Wilford",
		speech = "wilford", 		diet = {{FOODGROUP.OMNI}, {FOODTYPE.MEAT}}, wlkspd = 1.15,
		runspd = 1.17, 				talkercolor = {255,160,160}, 				
		
		specialfunction = nil
	},
	[FORMS.Fedoraplier] = {
		form = "Fedoraplier", 		regen = {0.2,10}, 							absorb = -0.2,
		hunger = 1.2, 				dapper = {"LARGE", 1},  					negaura = 2.5,
		nightdrain = 2.5, 			dmg = 0.6, 									scale = 0.95,
		icon = "markiplierfive", 	sound = "fedoraplier", 						--name = "Fedoraplier",
		speech = "fedoraplier", 	diet = {FOODGROUP.OMNI}, 					wlkspd = 0.75,
		runspd = 0.7, 				talkercolor = {200,200,200}, 				
		
		specialfunction = nil
	},
	[FORMS.Darkiplier] = {
		form = "Darkiplier", 		regen = {-0.1,20}, 							absorb = 0.15,
		hunger = 1, 				dapper = {"MED", -0.2}, 					negaura = 0.65,
		nightdrain = 0.6, 			dmg = 1.2, 									scale = 0.95,
		icon = "markiplierthree", 	sound = "darkiplier", 						--name = "Darkiplier",
		speech = "darkiplier", 		diet = {FOODGROUP.OMNI}, 					wlkspd = 0.95,
		runspd = 0.95, 				talkercolor = {60,60,60}, 					
		
		specialfunction = function(inst)
			inst:AddTag("monster")
			inst:AddTag("blacksmithimmune")
			inst.AnimState:SetBuild("markdark")
		end
	},
	[FORMS.Maniac] = {
		form = "Maniac", 			regen = {0.1,2}, 							absorb = 0.45,
		hunger = 0, 				dapper = {"MED", 0}, 						negaura = 0,
		nightdrain = 0, 			dmg = 2.5, 									scale = 1.05,
		icon = "markiplier", 	sound = "maniac", 							--name = "The Key Maniac",
		speech = "keymaniac",		diet = {{FOODTYPE.MEAT}, {FOODTYPE.MEAT}}, 	wlkspd = 1.5,
		runspd = 1.5, 				talkercolor = {220, 25, 25}, 				
		
		specialfunction = function(inst)
			ToManiac(inst)
		end
	}
}

local function ChangeStats(inst, state)
	local from = nil
	if state.specialfunction ~= nil then
		if (inst.MarkState == "Maniac" or inst.MarkState == "Darkiplier") and state.form == "Markiplier" then
			from = inst.MarkState
		end
	end
	if inst.MarkState ~= state.form then
		inst.MarkState = state.form
		inst.Transform:SetScale(state.scale, state.scale, state.scale)
		inst.MiniMapEntity:SetIcon(state.icon..".tex")
		inst.soundsname = state.sound
		--C.MARKIPLIER = C[string.upper(state.speech)] --temporary
		inst.speech = state.speech
		
		if inst.components and inst.components.talker ~= nil then
			inst.components.talker.colour = Vector3(state.talkercolor[1]/255,state.talkercolor[2]/255,state.talkercolor[3]/255)
		end

		if TheWorld.ismastersim then
			inst.components.health:StartRegen(state.regen[1], state.regen[2])
			inst.components.health.absorb = state.absorb
			inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE * state.hunger)
			inst.components.sanity.dapperness = (TUNING["DAPPERNESS_"..state.dapper[1]] * state.dapper[2])
			inst.components.sanity.neg_aura_mult = state.negaura
			inst.components.sanity.night_drain_mult = state.nightdrain
			inst.components.combat.damagemultiplier = state.dmg
			--inst.components.named:SetName(state.name)
			if #state.diet > 1 then inst.components.eater:SetDiet(state.diet[1], state.diet[2]) else inst.components.eater:SetDiet(state.diet) end
			inst.components.locomotor.walkspeed = (TUNING.WILSON_WALK_SPEED * state.wlkspd)
			inst.components.locomotor.runspeed = (TUNING.WILSON_RUN_SPEED * state.runspd)
		end
		
		if state.specialfunction ~= nil then
			local specialfn = state.specialfunction
			if from ~= nil then specialfn(inst, from) elseif inst.MarkState ~= "Markiplier" then specialfn(inst) end
		end
	end
end

local function RemoveHat(inst)
	local head = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD)
	if head then
		inst:DoTaskInTime(0.01, function()
			local item = inst.components.inventory:Unequip(EQUIPSLOTS.HEAD)
			inst.components.inventory:GiveItem(item)
		end)
	end
end

local function SpawnHero(inst)
	local HeroChance = MarkMod.HeroChance
	if TheWorld.state.isfullmoon then
		HeroChance = HeroChance * 1.3
	end
	if inst.components.sanity.current <= 60 and inst.components.sanity.current >= inst.darkiplierthreshold then
		HeroChance = HeroChance * 2
	end
	if math.random(1,100) <= HeroChance then
		local theta = math.random() * 2 * PI
		local pt = inst:GetPosition()
		local radius = math.random(30, 40)
		local offset = FindWalkableOffset(pt, theta, radius, 12, true)

		--[[if offset then
			local hero = SpawnPrefab("hoodedhero")
			local pos = pt + offset
			hero.Transform:SetPosition(pos:Get())
			inst.HeroSpawned = true
		end]]
	end
end

local function BecomeManiac(inst)
	if TheWorld.state.isnight --[[and not GetWorld():IsCave()]] then
		if inst.MarkState == "Markiplier" or inst.MarkState == "Wilford"  or inst.MarkState == "Fedoraplier" then
			inst.SoundEmitter:KillSound("talk")
			inst:SetStateManiac()
			inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.BLACKSMITH)
			
			if inst.components.playercontroller ~= nil then
				inst.components.playercontroller:Enable(true)
			end
		end
	else
		if inst.MarkState ~= "Darkipler" then
			inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.BLACKSMITH_CANCEL)
		elseif inst.MarkState == "Darkiplier" then
			inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.BLACKSMITH_CANCEL_DARK)
		end
		if inst.components.playercontroller ~= nil then
			inst.components.playercontroller:Enable(true)
		end
	end
end

local function BecomeMark(inst)
	if inst.HeroSpawned == true then inst.HeroSpawned = false end
	if inst.MarkState == "Maniac" then
		inst.components.locomotor:StopMoving()
		inst.components.playercontroller:Enable(false)
		inst:DoTaskInTime(0.2, function()
			inst:SetStateMarkiplier()
			if inst.components.health.currenthealth <= 15 then
				inst.components.health:DoDelta(10)
			end
			inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.BACK_TO_NORMAL)
		end)
	end
end

local function ManiacTimer(inst, force)
	inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.BLACKSMITH_WARN)
	inst.ManiacTimerGo = inst:DoPeriodicTask(1, function()
		if MarkMod.ManiacWarn == true then
			inst.ManiacTimer = inst.ManiacTimer - 1
		elseif MarkMod.ManiacWarn == false then
			inst.ManiacTimer = 0
		end
		if force and force == true then inst.ManiacTimer = 0 end
		if inst.ManiacTimer == 0 then
			inst.components.locomotor:StopMoving()
			if inst.components.playercontroller ~= nil then inst.components.playercontroller:Enable(false) end
			if inst.MarkState ~= "Maniac" and inst.MarkState ~= "Darkiplier" then
				if inst.MarkState == "Wilford" or inst.MarkState == "Fedoraplier" then
					inst:RemoveHat()
				end
				inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.BLACKSMITH_START)
				inst:DoTaskInTime(1.5, function()
					BecomeManiac(inst)
					if inst.HeroSpawned == false then
						SpawnHero(inst)
					end
				end)
			end
			inst.ManiacTimerGo:Cancel()
			inst.ManiacTimerGo = nil
		end
	end)
end

local function SetManiac(inst, force)
	local chance = MarkMod.ManiacChance
	if TheWorld.state.isfullmoon then
		chance = chance * 1.3
	end
	if inst.components.sanity.current <= 60 and inst.components.sanity.current >= inst.darkiplierthreshold then
		chance = chance * 2
	end
	if force and force == true then
		chance = 100
	end
	if math.random(1,100) <= chance then
		if TheWorld.state.isnight --[[and not GetWorld():IsCave()]] and inst.MarkState ~= "Maniac" and inst.MarkState ~= "Darkiplier" then
			inst.ManiacTimer = inst.ManiacTimer + math.random(7,12)
			ManiacTimer(inst, force)
		end
	end
end

local function Darkiplier(inst)
	if inst.MarkState ~= "Darkiplier" and inst.MarkState ~= "Maniac" and inst.components.sanity.current <= inst.darkiplierthreshold then
		if inst.MarkState == "Wilford" or inst.MarkState == "Fedoraplier" then
			inst:RemoveHat()
		end
		inst:SetStateDarkiplier()
		inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.ENTER_DARKIPLIER)
	elseif inst.MarkState == "Darkiplier" and inst.components.sanity.current >= inst.darkiplierthreshold then
		inst:SetStateMarkiplier()
		inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.LEAVE_DARKIPLIER)
	end
end

local function SpawnTim(inst)
	--if inst.TimPlush == nil then
		for k,v in pairs(inst.components.inventory.itemslots) do
			if v.prefab == "tinytimdevice" then
				v:SpawnTim(inst)
			end
		end
	--end
end

local function ondespawn(inst)
    --[[if inst.TimPlush ~= nil then
		inst.tim_save = inst.Tim:GetSaveRecord()
		inst.TimPlush:Remove()
	end
	if inst.Tim ~= nil then
		inst.timplush_save = inst.TimPlush:GetSaveRecord()
		inst.Tim:Remove()
	end]]
end

local function onsave(inst, data)
	data.MarkState = inst.MarkState
    --data.Tim = inst.tim_save
	--data.TimPlush = inst.timplush_save
	data.ManiacTimer = inst.ManiacTimer
	data.HeroSpawned = inst.HeroSpawned
end

local function onload(inst, data)
	if data then
		local MarkStateChooser = data.MarkState or "Markiplier"
		--Setting Mark to default state and build before applying his saved state
		if data.MarkState == "Darkiplier" or data.MarkState == "Maniac" then
			inst.AnimState:SetBuild("markiplier")
			if data.MarkState == "Maniac" then
				inst:SetStateGraph("SGwilson")
				inst.AnimState:SetBank("wilson")
			end
		end
		inst:SetStateMarkiplier()
		local ApplyState = {}
		ApplyState["Markiplier"] = inst.SetStateMarkiplier
		ApplyState["Wilford"] = inst.SetStateWilford
		ApplyState["Darkiplier"] = inst.SetStateDarkiplier
		ApplyState["Fedoraplier"] = inst.SetStateFedoraplier
		ApplyState["Maniac"] = inst.SetStateManiac
		ApplyState[MarkStateChooser](inst)
		
		inst.ManiacTimer = data.ManiacTimer
		if data.ManiacTimer >= 1 then
			ManiacTimer(inst, false)
		end
		inst.HeroSpawned = data.HeroSpawned
	end
	--local tim = nil
	--local plush = nil
	--if data.Tim ~= nil then tim = SpawnSaveRecord(data.Tim) end
	--if data.TimPlush ~= nil then plush = SpawnSaveRecord(data.TimPlush) end
	--if tim ~= nil and plush ~= nil then plush:sync(tim, inst) end
end

local function ToHuman(inst)
	inst:SetStateMarkiplier()
	inst:Darkiplier()
end

local function ToGhost(inst)
	inst.components.talker.colour = Vector3(1,1,1)
	if inst.MarkState == "Maniac" then
		ManiacMode(inst, false)
		inst.maniactoghost:push()
	end
end

local function getstatus(inst)
    if inst:HasTag("playerghost") then
        return "GHOST"
    elseif inst.hasRevivedPlayer then
        return "REVIVER"
    elseif inst.hasKilledPlayer then
        return "MURDERER"
    elseif inst.hasAttackedPlayer then
        return "ATTACKER"
	elseif inst.MarkState ~= "Markiplier" then
		return string.upper(inst.MarkState)
	end
end

local function ChangeForm(inst, form)
	inst.form = form
	inst.form_net:set(form)
	ChangeStats( inst, STATES[form] )
end
 
local function OnChangeForm(inst)
	inst.form = inst.form_net:value()
	ChangeStats( inst, STATES[inst.form] )
end

local clientfn = function(inst)
	inst.speech = "markiplier_base"
	inst:AddTag("ghostwithhat")
	inst:AddTag("markiplier")
	inst:AddTag("provenwarfstacher")
	inst.form = 1
	inst.form_net = net_byte(inst.GUID, "form", "form_dirty")
	if not TheWorld.ismastersim then
		inst:ListenForEvent("form_dirty", OnChangeForm)
		inst:ListenForEvent("maniactoghost", function(inst)
			--inst.components.talker.colour = Vector3(1,1,1)
			ManiacMode(inst, false)
		end)
	end
end

local serverfn = function(inst)
	inst.soundsname = "markiplier"
    inst.components.health:SetMaxHealth(170)
	inst.components.hunger:SetMax(160)
	inst.components.sanity:SetMax(200)
	
	inst:SetStateGraph("SGwilson")
	inst.AnimState:SetBuild("markiplier")
	inst.AnimState:SetBank("wilson")
	
	--inst:AddComponent("named")
	
	inst.ChatterBoxMultiState = true
	inst.GenericChatterBoxState = "markiplier"
	inst.ChatterBoxCheckState = function(inst)
		return string.lower(inst.MarkState) or inst.GenericChatterBoxState
	end
	inst.CustomChatterBoxGhostConvo = "markiplier_ghost"
	inst.ChatterBoxConvos = {
		tinyboxtim = {
			markiplier = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.converse["markiplier"],
			markiplier_ghost = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.converse["markiplier_ghost"],
			wilford = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.converse["wilford"],
			fedoraplier = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.converse["fedoraplier"],
			darkiplier = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.converse["darkiplier"],
			maniac = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.converse["maniac"],
		},
		googleirl = {
			markiplier = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.converse["markiplier"],
			markiplier_ghost = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.converse["ghost"],
			wilford = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.converse["wilford"],
			fedoraplier = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.converse["fedoraplier"],
			darkiplier = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.converse["darkiplier"],
			maniac = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.converse["maniac"],
		},
		septiceyesam = {
			markiplier = {},
			markiplier_ghost = {},
			wilford = {},
			fedoraplier = {},
			darkiplier = {},
			maniac = {},
		},
	}
	
	inst.maniactoghost = net_event(inst.GUID, "maniactoghost")
	inst.SetStateMarkiplier = function(inst) ChangeForm(inst, FORMS.Markiplier) end
	inst.SetStateWilford = function(inst) ChangeForm(inst, FORMS.Wilford) end
	inst.SetStateFedoraplier = function(inst) ChangeForm(inst, FORMS.Fedoraplier) end
	inst.SetStateDarkiplier = function(inst) ChangeForm(inst, FORMS.Darkiplier) end
	inst.SetStateManiac = function(inst) ChangeForm(inst, FORMS.Maniac) end
	inst:SetStateMarkiplier()
	
	inst.RemoveHat = RemoveHat
	inst.darkiplierthreshold = MarkMod.DarkThresh
	inst.Darkiplier = Darkiplier
	inst:DoPeriodicTask(5, function() if inst.MarkState ~= "Maniac" and not inst:HasTag("playerghost") then inst:Darkiplier() end end)
	
	inst.ManiacTimer = 0
	inst.HeroSpawned = false
	inst.SetManiac = SetManiac
    --inst:ListenForEvent( "nighttime", function() if not inst:HasTag("playerghost") then SetManiac(inst, false) end end, TheWorld )
	--inst:ListenForEvent( "daytime", function() if not inst:HasTag("playerghost") then BecomeMark(inst) end end, TheWorld )
	
	--inst.components.inspectable.getstatus = getstatus
	
	--inst.TimPlush = nil
	--inst.Tim = nil
	inst:DoTaskInTime(0.1, function()
		SpawnTim(inst)
	end)
	--inst.tim_save = nil
	--inst.timplush_save = nil
	
	inst:ListenForEvent("ms_respawnedfromghost", ToHuman)
    inst:ListenForEvent("ms_becameghost", ToGhost)
	inst.OnDespawn = ondespawn
	inst.OnSave = onsave
	inst.OnLoad = onload
end

return MakePlayerCharacter("markiplier", prefabs, assets, clientfn, serverfn, start_inv)