require "prefabutil"
local brain = require "brains/tinyboxtimbrain"

local assets = {
	Asset("SOUND", "sound/markimod.fsb"),
    Asset("ANIM", "anim/tinyboxtim.zip"),
	Asset("ANIM", "anim/tinywarfboxtim.zip"),
	Asset("ANIM", "anim/euphoricboxtim.zip"),
	Asset("ANIM", "anim/euphoricwarfboxtim.zip")
}

local prefabs = { "tinytimdevice" }

local TimWidget = {
	widget = {
		slotpos = {},
		animbank = "ui_cookpot_1x4",
		animbuild = "ui_cookpot_1x4",
		pos = Vector3(0, 200, 0),
		side_align_tip = 160,
	},
	type = "chest",
}

for i = 0,4 do
    table.insert(TimWidget.widget.slotpos, Vector3(0,120-(70*i),0))
end

local function OnWidgetUpdate(inst)
    inst.replica.container:WidgetSetup(inst.prefab, TimWidget)
end

local FORMS = {tinyboxtim = 1, tinywarfboxtim = 2, euphoricboxtim = 3, euphoricwarfboxtim = 4}
local STATES = {
	[FORMS.tinyboxtim] = { name = "tinyboxtim", stringname = "Tiny Box Tim", sanity = 0.1, speech_color = {150, 100, 50} },
	[FORMS.tinywarfboxtim] = { name = "tinywarfboxtim", stringname = "Tiny WarfBox Tim", sanity = 0.5, speech_color = {255, 160, 160} },
	[FORMS.euphoricboxtim] = { name = "euphoricboxtim", stringname = "Euphoric Box Tim", sanity = 0.7, speech_color = {200, 200, 200} },
	[FORMS.euphoricwarfboxtim] = { name = "euphoricwarfboxtim", stringname = "Euphoric WarfBox Tim", sanity = 1, speech_color = {255, 200, 200} }
}

local function ChangeTim(inst, state)
	inst.AnimState:SetBank(state.name)
	inst.AnimState:SetBuild(state.name)
	inst.MiniMapEntity:SetIcon(state.name..".tex")
	if inst.components and inst.components.talker ~= nil then
		inst.components.talker.colour = Vector3(state.speech_color[1]/255,state.speech_color[2]/255,state.speech_color[3]/255)
	end
	if TheWorld.ismastersim then
		inst.TimState = state.name
		inst.components.named:SetName(state.stringname)
		inst.components.sanityaura.aurafn = function() return TUNING.SANITYAURA_TINY * state.sanity end
		if inst.sg:HasStateTag("open") then inst.sg:GoToState("open") else inst.sg:GoToState("idle") end
	end
end

local function ChangeForm(inst, form)
	inst.form = form
	inst.form_net:set(form)
	ChangeTim( inst, STATES[form] )
end

local function OnChangeForm(inst)
	inst.form = inst.form_net:value()
	ChangeTim( inst, STATES[inst.form] )
end

local function StateChecker(inst)
	local WarfMe = false
	local FancyMe = false
	if inst.components.container:Has("warfstache", 1) then
		WarfMe = true
	end
	if inst.components.container:Has("fedora", 1) then
		FancyMe = true
	end
	if WarfMe == true and FancyMe == false and (inst.Warfified == false or (inst.Warfified == true and inst.Euphoric == true)) then
		if inst.Euphoric == true then inst.Euphoric = false end
		inst.Warfified = true
		ChangeForm(inst, FORMS.tinywarfboxtim)
	elseif FancyMe == true and WarfMe == false and (inst.Euphoric == false or (inst.Warfified == true and inst.Euphoric == true)) then
		if inst.Warfified == true then inst.Warfified = false end
		inst.Euphoric = true
		ChangeForm(inst, FORMS.euphoricboxtim)
	elseif WarfMe == true and FancyMe == true and (inst.Warfified == false or inst.Euphoric == false) then
		if inst.Warfified == false then inst.Warfified = true end
		if inst.Euphoric == false then inst.Euphoric = true end
		ChangeForm(inst, FORMS.euphoricwarfboxtim)
	elseif WarfMe == false and FancyMe == false and (inst.Warfified == true or inst.Euphoric == true) then
		if inst.Warfified == true then inst.Warfified = false end
		if inst.Euphoric == true then inst.Euphoric = false end
		ChangeForm(inst, FORMS.tinyboxtim)
	end
end

local function OnOpen(inst)
	inst.sg:GoToState("open")
	inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.on_open))
end 

local function OnClose(inst) 
    inst.sg:GoToState("close")
	inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.on_close))
end

local function getstatus(inst)
		if inst.TimState == "tinywarfboxtim" 	 then return "WARFBOX"
	elseif inst.TimState == "euphoricboxtim" 	 then return "EUPHORICBOX"
	elseif inst.TimState == "euphoricwarfboxtim" then return "WARFORICBOX"
	elseif inst.TimState == "tinyboxtim" 		 then return "GENERIC"
end end

local function create_tinyboxtim()
    local inst = CreateEntity()
	inst.entity:AddNetwork()
	
	inst.entity:AddTransform()
	inst.Transform:SetScale(0.25,0.25,0.25)
	inst.Transform:SetFourFaced()
	
	inst.entity:AddSoundEmitter()
	
    inst.entity:AddAnimState()
	inst.AnimState:SetBank("tinyboxtim_idle")
    inst.AnimState:SetBuild("tinyboxtim")
	inst.AnimState:PlayAnimation("idle")
	
	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon("tinyboxtim.tex")
	inst.MiniMapEntity:SetCanUseCache(false)
	
    inst.entity:AddDynamicShadow()
    inst.DynamicShadow:SetSize( 1, 1 )
	
	MakeCharacterPhysics(inst, 75, .5)
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.WORLD)
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:CollidesWith(COLLISION.CHARACTERS)
	
	inst:AddTag("companion")
    inst:AddTag("character")
    inst:AddTag("tinyboxtim")
    inst:AddTag("notraptrigger")
	inst:AddTag("noauradamage")
	
	inst.form = 1
	inst.form_net = net_byte(inst.GUID, "form", "form_dirty")
	inst._widgetupdate = net_bool(inst.GUID, "_widgetupdate", "onwidgetupdate")
	
	inst:AddComponent("talker")
	inst.components.talker.colour = Vector3(150/255,100/255,50/255)
    inst.components.talker.font = TALKINGFONT
    inst.components.talker.fontsize = 28
	inst.components.talker.offset = Vector3(0,-500,0)

	inst.entity:SetPristine()
	if not TheWorld.ismastersim then
		inst:ListenForEvent("form_dirty", OnChangeForm)
		inst:ListenForEvent("onwidgetupdate", OnWidgetUpdate)
		return inst
	end
	
    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 55
    inst.components.locomotor.runspeed = 55
	inst.components.locomotor.fasteronroad = true
	
	inst:SetStateGraph("SGtinyboxtim")
    inst.sg:GoToState("idle")
	
	inst.NonCharacterChatterer = true
	inst.ChatterBoxCondition = function(inst)
		if inst.components.container and not inst.components.container:IsOpen() and not inst.sg:HasStateTag("sleeping") then
			return true
		end
	end 
	inst.ClosingChatterBox = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.cancelconvo
	inst.SimplifiedConvoTables = true
	inst.SimpleConvo = {}
	inst.SimpleConvo.tinyboxtim = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.converse["tinyboxtim"]
	inst.SimpleConvo.googleirl = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.converse["tinyboxtim"]
	inst.SimpleConvo.septiceyesam = {}
	
	inst:AddComponent("chatterbox")
	inst.components.chatterbox:SetSpeech(STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH)
	local commonstringtable = inst.components.chatterbox.speech
	inst.components.chatterbox:SetCondition(function()
		if not inst.sg:HasStateTag("talking") and inst.components.chatterbox.run_convo == false then
			if not inst.sg:HasStateTag("sleeping") then return true
			else return false end
		else return false end
	end)
	inst.components.chatterbox:SetIdleChat(function()
		if inst.components.chatterbox:CheckSurroundings() ~= "cave" then
			if not inst.sg:HasStateTag("open") then
				if math.random(1,100) <= 50 then
					local Time = inst.components.chatterbox:CheckSurroundings()
					if commonstringtable[Time] then
						if Time == "fullmoon" then
							if math.random(1,100) <= 20 then
							inst.components.talker:Say(GetRandomItem(commonstringtable["night"])) else
							inst.components.talker:Say(GetRandomItem(commonstringtable[Time])) end
						else inst.components.talker:Say(GetRandomItem(commonstringtable[Time])) end
					else inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"])) end
				else inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"])) end
			else inst.components.talker:Say(GetRandomItem(commonstringtable["open"])) end
		else
			if math.random(1,100) <= 40 then inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"]))
			else inst.components.talker:Say(GetRandomItem(commonstringtable["cave_chat"])) end
		end
	end)
	inst.components.chatterbox:SetItemTracker({
		giants = {"moose", "dragonfly", "deerclops", "bearger"},
		teleportatopieces = {"teleportato_base", "teleportato_crank", "teleportato_ring", "teleportato_potato", "teleportato_box"},
		pig_king = {"pigking", "pigking"},
		blacksmithsafe = {"lockedblacksmithsafe", "lockedblacksmithsafe"},
		patient = {"patientbab", "heartbox"}
	})
	inst.components.chatterbox:EnableConversing(50, 150, 70, 20)
	inst.components.chatterbox:IdleChat(10, 25, 60)
	inst.components.chatterbox:Encourage(1, 5, 80, "markiplier", 20)
	inst.components.chatterbox:LookForItems(10, 40, 65, 75)
	
    inst:AddComponent("follower")
	
    inst:AddComponent("knownlocations")
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
	
	inst:AddComponent("sanityaura")
	
	inst:AddComponent("named")
	
	--inst.MyPlush = nil
	--inst.MyMark = nil
	inst.Warfified = false
	inst.Euphoric = false
	inst.TimState = "tinyboxtim"
	
    inst:AddComponent("container")
    inst.components.container:SetNumSlots(5)
    inst.components.container.onopenfn = OnOpen
    inst.components.container.onclosefn = OnClose
	inst.components.container:WidgetSetup(inst.prefab, TimWidget)
	inst._widgetupdate:set(true)
	inst:DoPeriodicTask(0.1, function()
		StateChecker(inst)
		if not inst.components.follower.leader and not inst:HasTag("tinyboxtimnoleader") then
			inst:AddTag("tinyboxtimnoleader")
		elseif inst.components.follower.leader ~= nil and inst:HasTag("tinyboxtimnoleader") then
			inst:RemoveTag("tinyboxtimnoleader")
		end
	end)
	
	MakeHauntableGoToState(inst, "scaredtaunt", nil, 64 * FRAMES * 1.2)
    inst:SetBrain(brain)
	
    return inst
end

return Prefab( "common/tinyboxtim", create_tinyboxtim, assets, prefabs) 