local function MakeStache(dt)
	local assets = {
		Asset("ANIM", "anim/"..dt.name..".zip"),
		Asset("ATLAS", "images/inventoryimages/"..dt.name..".xml")
	}

	local function onequip(inst, owner) 
		owner.AnimState:OverrideSymbol("swap_hat", dt.name, "swap_hat")
		owner.AnimState:Show("HAT")
		if dt.name == "fedora" then
			owner.AnimState:Show("HAT_HAIR")
			owner.AnimState:Hide("HAIR_NOHAT")
			owner.AnimState:Hide("HAIR")
			if owner:HasTag("player") then
				owner.AnimState:Hide("HEAD")
				owner.AnimState:Show("HEAD_HAT")
			end
		else
			if not owner:HasTag("provenwarfstacher") then
				inst:DoTaskInTime(0.1, function()
					local head = owner.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD)
					if head then
						if owner.components.talker then owner.components.talker:Say(GetString(owner, "ANNOUNCE_NOWARFSTACHE")) end
						local item = owner.components.inventory:Unequip(EQUIPSLOTS.HEAD)
						owner.components.inventory:GiveItem(item)
					end
				end)
			end
		end
		if owner.prefab == "markiplier" then
			if owner.MarkState ~= "Darkiplier" then
				if dt.name == "warfstache" then owner:SetStateWilford() else owner:SetStateFedoraplier() end
				if TheWorld.ismastersim and owner.components.talker then owner.components.talker:Say(GetRandomItem(dt.enterspeech)) end
			elseif owner.MarkState == "Darkiplier" then
				owner:DoTaskInTime(0.1, function(owner)
					owner:RemoveHat(owner)
					if owner.components.talker then owner.components.talker:Say(STRINGS.MARKIMODSTRINGS.CANCEL_WILF_DARK) end
				end)
			end
		end
	end

	local function onunequip(inst, owner) 
		if owner.prefab == "markiplier" then
			if owner.MarkState == dt.state then
				owner:SetStateMarkiplier()
			end
		end
		owner.AnimState:ClearOverrideSymbol("swap_hat")
		owner.AnimState:Hide("HAT")
		if dt.name == "fedora" then
			owner.AnimState:Hide("HAT_HAIR")
			owner.AnimState:Show("HAIR_NOHAT")
			owner.AnimState:Show("HAIR")
			if owner:HasTag("player") then
				owner.AnimState:Show("HEAD")
				owner.AnimState:Hide("HEAD_HAT")
			end
		end
	end

	local function fn(Sim)
		local inst = CreateEntity()
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		inst.entity:AddMiniMapEntity()
		inst.entity:AddNetwork()
		
		inst.MiniMapEntity:SetIcon(dt.name..".tex")
		MakeInventoryPhysics(inst)
		
		inst:AddTag("hat")
		inst:AddTag(dt.name)
		
		inst.AnimState:SetBank(dt.bank)
		inst.AnimState:SetBuild(dt.name)
		inst.AnimState:PlayAnimation("anim")
		
		inst.entity:SetPristine()
		if not TheWorld.ismastersim then
			return inst
		end
		MakeHauntableLaunch(inst)
		
		inst:AddComponent("tradable")
		
		inst:AddComponent("inspectable")
		
		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.atlasname = "images/inventoryimages/"..dt.name..".xml"
		
		inst:AddComponent("equippable")
		inst.components.equippable.equipslot = EQUIPSLOTS.HEAD
		inst.components.equippable:SetOnEquip( onequip )
		inst.components.equippable:SetOnUnequip( onunequip )
		inst.components.equippable.dapperness = dt.dapperness
		
		if dt.name == "fedora" then
			inst:AddComponent("waterproofer")
			inst.components.waterproofer:SetEffectiveness(TUNING.WATERPROOFNESS_SMALL)
			inst.components.equippable.insulated = true
		end
		
		inst.WarFX = nil
		inst:DoPeriodicTask(0.1, function()
			if inst.WarFX == nil and inst.components.inventoryitem.owner == nil then
				local WarFX = SpawnPrefab("warfx")
				WarFX:Follow(inst)
			end
		end)
		
		return inst
	end

	return Prefab( "common/inventory/"..dt.name, fn, assets)
end

local data = {
	{name="warfstache", bank="beehat", enterspeech=STRINGS.MARKIMODSTRINGS.MARK_WILFORD, dapperness=TUNING.DAPPERNESS_SMALL, state="Wilford"},
	{name="fedora", bank="rainhat", enterspeech=STRINGS.MARKIMODSTRINGS.ENTER_FEDORAPLIER, dapperness=TUNING.DAPPERNESS_LARGE, state="Fedoraplier"}
}

local ret = {}
for k,v in pairs(data) do
	table.insert(ret, MakeStache(v))
end

return unpack(ret)