local function DoFoleySounds(inst)
    for k, v in pairs(inst.components.inventory.equipslots) do
        if v.foleysound ~= nil then
            inst.SoundEmitter:PlaySound(v.foleysound, nil, nil, true)
        end
    end
    if inst.foleysound ~= nil then
        inst.SoundEmitter:PlaySound(inst.foleysound, nil, nil, true)
    end
end

local function DoYawnSound(inst)
    inst.SoundEmitter:PlaySound("dontstarve/characters/wilson/yawn") --CHANGE
end

local function DoTalkSound(inst)
	inst.SoundEmitter:PlaySound("dontstarve/characters/wilson/talk_LP") --CHANGE
end

local actionhandlers = {
    ActionHandler(ACTIONS.CHOP, "work"),
    ActionHandler(ACTIONS.MINE, "work"),
    ActionHandler(ACTIONS.DIG, "work"),
    ActionHandler(ACTIONS.HAMMER, "work"),
    ActionHandler(ACTIONS.ATTACK, "work"),
   ActionHandler(ACTIONS.EAT,
	function(inst, action)
		if inst.sg:HasStateTag("busy") then
			return
		end
		local obj = action.target or action.invobject
		if obj == nil or obj.components.edible == nil then
			return
		elseif not inst.components.eater:PrefersToEat(obj) then
			inst:PushEvent("wonteatfood", { food = obj })
			return
		end
		return (obj.components.edible.foodtype == FOODTYPE.MEAT and "eat") or "quickeat"
	end),
}

local events = {
	EventHandler("locomote", function(inst, data)
        if inst.sg:HasStateTag("busy") then
            return
        end
        local is_moving = inst.sg:HasStateTag("moving")
        local should_move = inst.components.locomotor:WantsToMoveForward()

        if is_moving and not should_move then
            inst.sg:GoToState("run_stop")
        elseif not is_moving and should_move then
            inst.sg:GoToState("run_start")
        elseif data.force_idle_state and not (is_moving or should_move or inst.sg:HasStateTag("idle")) then
            inst.sg:GoToState("idle")
        end
    end),
	EventHandler("ontalk", function(inst, data)
        if inst.sg:HasStateTag("idle") and not inst.sg:HasStateTag("notalking") then
            inst.sg:GoToState("talk", data.noanim)
        end
    end),
	EventHandler("attacked", function(inst, data) if not inst.components.health:IsDead() then
		if data.attacker ~= nil and data.attacker:HasTag("insect") and not inst.sg:HasStateTag("idle") then
			inst.SoundEmitter:PlaySound("dontstarve/wilson/hit") --CHANGE
		elseif data.stimuli == "electric" and not inst.components.inventory:IsInsulated() then
			inst.sg:GoToState("electrocute")
		else
			inst.sg:GoToState("hit")
		end
	end end),
	EventHandler("death", function(inst)
		if inst.components.playercontroller ~= nil then
            inst.components.playercontroller:Enable(false)
        end
		
        inst.sg:GoToState("death")
        inst.SoundEmitter:PlaySound("dontstarve/wilson/death") --CHANGE

        if HUMAN_MEAT_ENABLED then
            for i = 1,3 do inst.components.inventory:GiveItem(SpawnPrefab("humanmeat")) end
        end
        inst.components.inventory:DropEverything(true)
	end),
	EventHandler("yawn", function(inst, data)
		if not (inst.components.health:IsDead() or inst.sg:HasStateTag("frozen")) then
			inst.sg:GoToState("yawn", data)
		end
	end),
    --[[EventHandler("emote", function(inst, data)
		if not (inst.sg:HasStateTag("busy")) then
			inst.sg:GoToState("emote", data)
		end
	end),]]
    EventHandler("freeze", function(inst)
		if inst.components.health ~= nil and not inst.components.health:IsDead() then
			inst.sg:GoToState("frozen")
		end
	end),
    EventHandler("wonteatfood", function(inst)
		if inst.components.health ~= nil and not inst.components.health:IsDead() then
			inst.sg:GoToState("refuseeat")
		end
	end)
}


local states = {
	--[[State{
		name = "from_mark",
		tags = {},
		onenter = function(inst)
			
		end,
		
		events = {}
	},
	State{
		name = "to_mark",
		tags = {},
		onenter = function(inst)
			
		end,
		
		events = {}
	},]]
	State{
		name = "talk",
		tags = {"idle", "talking"},
		onenter = function(inst, noanim)
			if not noanim then
                inst.AnimState:PlayAnimation("talk", true)
            end
            DoTalkSound(inst)
            inst.sg:SetTimeout(1.5 + math.random() * .5)
		end,
		
        ontimeout = function(inst)
            inst.sg:GoToState("idle")
        end,
		
		events = {
            EventHandler("donetalking", function(inst)
                inst.sg:GoToState("idle")
            end),
        },

        onexit = function(inst)
            inst.SoundEmitter:KillSound("talk")
        end
	},
	State{
		name = "idle",
		tags = {"idle", "canrotate"},
		onenter = function(inst)
			inst.Physics:Stop()
			inst.AnimState:PlayAnimation("idle_loop")
		end,
		
		events = {EventHandler("animover", function(inst) inst.sg:GoToState("idle") end)}
	},
	State{
		name = "electrocute",
		tags = {"busy"},
		onenter = function(inst)
			if inst.components.freezable ~= nil and inst.components.freezable:IsFrozen() then
                inst.components.freezable:Unfreeze()
            end
			inst.components.locomotor:Stop()
            inst:ClearBufferedAction()

            inst.fx = SpawnPrefab("shock_fx")
            inst.fx.Transform:SetRotation(inst.Transform:GetRotation())
            inst.fx.Transform:SetPosition(inst.Transform:GetWorldPosition())

            if not inst:HasTag("electricdamageimmune") then
                inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
                inst.Light:Enable(true)
            end
			inst.AnimState:PlayAnimation("shock")
            inst.AnimState:PushAnimation("shock_pst", false)

            inst.SoundEmitter:PlaySound("dontstarve/wilson/hit") --CHANGE

            if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:RemotePausePrediction()
            end
            inst.sg:SetTimeout(8 * FRAMES + inst.AnimState:GetCurrentAnimationLength())
		end,
		
		events = {
			EventHandler("animover", function(inst)
                if inst.fx ~= nil then
                    if not inst:HasTag("electricdamageimmune") then
                        inst.Light:Enable(false)
                        inst.AnimState:ClearBloomEffectHandle()
                    end
                    inst.fx:Remove()
                    inst.fx = nil
                end
            end),

            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
		}
	},
	State{
		name = "hit",
		tags = {"busy"},
		onenter = function(inst)
			inst.components.locomotor:Stop()
            inst:ClearBufferedAction()

            inst.AnimState:PlayAnimation("hurt")
			if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:RemotePausePrediction(5)
            end
			inst.sg:SetTimeout(5 * FRAMES)

            inst.SoundEmitter:PlaySound("dontstarve/wilson/hit") --CHANGE
		end,
		
		events = {EventHandler("animover", function(inst) inst.sg:GoToState("idle") end)}
	},
	State{
		name = "death",
		tags = {"busy", "pausepredict", "nomorph"},
		onenter = function(inst)
			assert(inst.deathcause ~= nil, "Entered death state without cause.")
			inst.components.locomotor:Stop()
            inst.components.locomotor:Clear()
            inst:ClearBufferedAction()

            inst.AnimState:PlayAnimation("dead_pre")
			inst.AnimState:PushAnimation("dead_loop")
            inst.components.burnable:Extinguish()

            if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:RemotePausePrediction()
            end

            inst.sg:ClearBufferedEvents()
		end,
		
        onexit = function(inst)
            assert(false, "Left death state.")
        end,
		
		events = {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst:PushEvent(inst.ghostenabled and "makeplayerghost" or "playerdied", { skeleton = true })
                end
            end),
        }
	},
	State{
		name = "frozen",
		tags = {"busy", "frozen", "nopredict"},
		onenter = function(inst)
			inst.components.locomotor:Stop()
            inst:ClearBufferedAction()

            inst.AnimState:OverrideSymbol("swap_frozen", "frozen", "frozen")
            inst.AnimState:PlayAnimation("frozen")
            inst.SoundEmitter:PlaySound("dontstarve/common/freezecreature")

            inst.components.inventory:Hide()
            inst:PushEvent("ms_closepopups")
            if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:EnableMapControls(false)
                inst.components.playercontroller:Enable(false)
            end
		end,
		
		events = {
            EventHandler("onthaw", function(inst)
                inst.sg.statemem.isstillfrozen = true
                inst.sg:GoToState("thaw")
            end),
            EventHandler("unfreeze", function(inst)
                inst.sg:GoToState("hit")
            end),
        },
		
		onexit = function(inst)
            if not inst.sg.statemem.isstillfrozen then
                inst.components.inventory:Show()
                if inst.components.playercontroller ~= nil then
                    inst.components.playercontroller:EnableMapControls(true)
                    inst.components.playercontroller:Enable(true)
                end
            end
            inst.AnimState:ClearOverrideSymbol("swap_frozen")
        end
	},
	State{
		name = "thaw",
		tags = {"busy", "thawing", "nopredict"},
		onenter = function(inst)
			inst.components.locomotor:Stop()
            inst:ClearBufferedAction()

            inst.AnimState:OverrideSymbol("swap_frozen", "frozen", "frozen")
            inst.AnimState:PlayAnimation("frozen_loop_pst", true)
            inst.SoundEmitter:PlaySound("dontstarve/common/freezethaw", "thawing")

            inst.components.inventory:Hide()
            inst:PushEvent("ms_closepopups")
            if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:EnableMapControls(false)
                inst.components.playercontroller:Enable(false)
            end
		end,
		
		events = {
            EventHandler("unfreeze", function(inst)
                inst.sg:GoToState("hit")
            end),
        },
		
		onexit = function(inst)
            inst.components.inventory:Show()
            if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:EnableMapControls(true)
                inst.components.playercontroller:Enable(true)
            end
            inst.SoundEmitter:KillSound("thawing")
            inst.AnimState:ClearOverrideSymbol("swap_frozen")
        end
	},
	State{
		name = "yawn",
		tags = {"busy", "yawn", "pausepredict"},
		onenter = function(inst, data)
			inst.components.locomotor:Stop()
            inst:ClearBufferedAction()

            if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:RemotePausePrediction()
            end
			
			inst.AnimState:PlayAnimation("yawn")
		end,
		
        timeline = { TimeEvent(18 * FRAMES, DoYawnSound) },
		
		events = {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:RemoveStateTag("yawn")
                    inst.sg:GoToState("idle")
                end
            end),
        }
	},
	State{
		name = "quickeat",
		tags = {"busy"},
		onenter = function(inst)
			inst.components.locomotor:Stop()

            if feed ~= nil then
                inst.components.locomotor:Clear()
                inst:ClearBufferedAction()
                inst.sg.statemem.feed = feed
                inst.sg:AddStateTag("pausepredict")
                if inst.components.playercontroller ~= nil then
                    inst.components.playercontroller:RemotePausePrediction()
                end
            elseif inst:GetBufferedAction() then
                feed = inst:GetBufferedAction().invobject
            end

            if feed == nil or
                feed.components.edible == nil or
                feed.components.edible.foodtype ~= FOODTYPE.GEARS then
                inst.SoundEmitter:PlaySound("dontstarve/wilson/eat", "eating")
            end

            inst.AnimState:PlayAnimation("eating")

            inst.components.hunger:Pause()
		end,
		
        timeline = {
            TimeEvent(12 * FRAMES, function(inst)
                if inst.sg.statemem.feed ~= nil then
                    inst.components.eater:Eat(inst.sg.statemem.feed)
                else
                    inst:PerformBufferedAction()
                end
                inst.sg:RemoveStateTag("busy")
                inst.sg:RemoveStateTag("pausepredict")
            end),
        },

        events = {
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },

        onexit = function(inst)
            inst.SoundEmitter:KillSound("eating")
            inst.components.hunger:Resume()
            if inst.sg.statemem.feed ~= nil and inst.sg.statemem.feed:IsValid() then
                inst.sg.statemem.feed:Remove()
            end
        end
	},
	State{
		name = "eat_large",
		tags = {"busy"},
		onenter = function(inst, feed)
			inst.components.locomotor:Stop()

            if feed ~= nil then
                inst.components.locomotor:Clear()
                inst:ClearBufferedAction()
                inst.sg.statemem.feed = feed
                inst.sg:AddStateTag("pausepredict")
                if inst.components.playercontroller ~= nil then
                    inst.components.playercontroller:RemotePausePrediction()
                end
            elseif inst:GetBufferedAction() then
                feed = inst:GetBufferedAction().invobject
            end

            if feed == nil or
                feed.components.edible == nil or
                feed.components.edible.foodtype ~= FOODTYPE.GEARS then
                inst.SoundEmitter:PlaySound("dontstarve/wilson/eat", "eating")
            end
            
            inst.AnimState:PlayAnimation("eat_large")

            inst.components.hunger:Pause()
		end,
		
        timeline = {
            TimeEvent(28 * FRAMES, function(inst)
                if inst.sg.statemem.feed ~= nil then
                    inst.components.eater:Eat(inst.sg.statemem.feed)
                else
                    inst:PerformBufferedAction() 
                end
            end),

            TimeEvent(30 * FRAMES, function(inst) 
                inst.sg:RemoveStateTag("busy")
                inst.sg:RemoveStateTag("pausepredict")
            end),

            TimeEvent(70 * FRAMES, function(inst) 
                inst.SoundEmitter:KillSound("eating")    
            end),
        },

        events = {
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },

        onexit = function(inst)
            inst.SoundEmitter:KillSound("eating")
            inst.components.hunger:Resume()
            if inst.sg.statemem.feed ~= nil and inst.sg.statemem.feed:IsValid() then
                inst.sg.statemem.feed:Remove()
            end
        end
	},
	--[[State{
		name = "emote",
		tags = {},
		onenter = function(inst)
			
		end,
		
		events = {}
	},]]
	State{
		name = "refuseeat",
		tags = {"busy", "pausepredict"},
		onenter = function(inst)
			inst.components.locomotor:Stop()
            inst.components.locomotor:Clear()
            inst:ClearBufferedAction()

            inst.AnimState:PlayAnimation("refuseeat")

            if inst.components.playercontroller ~= nil then
                inst.components.playercontroller:RemotePausePrediction()
            end
            inst.sg:SetTimeout(22 * FRAMES)
		end,
		
        ontimeout = function(inst)
            inst.sg:GoToState("idle", true)
        end
	},
	State{
		name = "work",
		tags = {"working"},
		onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.sg.statemem.action = inst:GetBufferedAction()
            inst.AnimState:PlayAnimation("punch_"..math.random(1,3))
            inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
		end,
		
		timeline = {
            TimeEvent(6 * FRAMES, function(inst)
                if inst.sg.statemem.action ~= nil then
                    local target = inst.sg.statemem.action.target
                    if target ~= nil and target:IsValid() then
                        if inst.sg.statemem.action.action == ACTIONS.MINE then
                            SpawnPrefab("mining_fx").Transform:SetPosition(target.Transform:GetWorldPosition())
                            inst.SoundEmitter:PlaySound(target:HasTag("frozen") and "dontstarve_DLC001/common/iceboulder_hit" or "dontstarve/wilson/use_pick_rock")
                        elseif inst.sg.statemem.action.action == ACTIONS.HAMMER then
                            inst.SoundEmitter:PlaySound("dontstarve/wilson/hit")
                        elseif inst.sg.statemem.action.action == ACTIONS.DIG then
                            SpawnPrefab("shovel_dirt").Transform:SetPosition(target.Transform:GetWorldPosition())
                        end
                    end
                end
                inst:PerformBufferedAction()
            end),
            
            TimeEvent(8 * FRAMES, function(inst)
                if inst.sg.statemem.action == nil or
                    inst.sg.statemem.action.action == nil or
                    inst.components.playercontroller == nil then
                    return
                end
                local rmb = inst.sg.statemem.action.action == ACTIONS.HAMMER
                if rmb then
                    if not inst.components.playercontroller:IsAnyOfControlsPressed(
                            CONTROL_SECONDARY,
                            CONTROL_CONTROLLER_ALTACTION) then
                        return
                    end
                elseif not inst.components.playercontroller:IsAnyOfControlsPressed(
                            CONTROL_PRIMARY,
                            CONTROL_ACTION,
                            CONTROL_CONTROLLER_ACTION) then
                    return
                end
                if inst.sg.statemem.action:IsValid() and
                    inst.sg.statemem.action.target ~= nil and
                    inst.sg.statemem.action.target.components.workable ~= nil and
                    inst.sg.statemem.action.target.components.workable:CanBeWorked() and
                    inst.sg.statemem.action.target.components.workable:GetWorkAction() == inst.sg.statemem.action.action then
                    inst:ClearBufferedAction()
                    inst:PushBufferedAction(inst.sg.statemem.action)
                end
            end),
        },
		
		events = {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        }
	},
	State{
		name = "run_start",
		tags = {"moving", "running", "canrotate", "autopredict"},
		onenter = function(inst)
			inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("walking")
            inst.sg.mem.footsteps = 0
		end,
		
		onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,
		
		timeline = {
            TimeEvent(4 * FRAMES, function(inst)
                PlayFootstep(inst, nil, true)
                DoFoleySounds(inst)
            end),
        },
		
		events = {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("run_loop")
                end
            end),
        }
	},
	State{
		name = "run_loop",
		tags = {"moving", "running", "canrotate", "autopredict"},
		onenter = function(inst)
			inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("walking", true)
            inst.sg:SetTimeout(inst.AnimState:GetCurrentAnimationLength())
		end,
		
        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        timeline = {
            TimeEvent(7 * FRAMES, function(inst)
                if inst.sg.mem.footsteps > 3 then
                    PlayFootstep(inst, .6, true)
                else
                    inst.sg.mem.footsteps = inst.sg.mem.footsteps + 1
                    PlayFootstep(inst, 1, true)
                end
                DoFoleySounds(inst)
            end),
            TimeEvent(15 * FRAMES, function(inst)
                if inst.sg.mem.footsteps > 3 then
                    PlayFootstep(inst, .6, true)
                else
                    inst.sg.mem.footsteps = inst.sg.mem.footsteps + 1
                    PlayFootstep(inst, 1, true)
                end
                DoFoleySounds(inst)
            end),
        },
		
        ontimeout = function(inst)
            inst.sg:GoToState("run")
        end,
	},
	State{
		name = "run_stop",
		tags = {"canrotate", "idle", "autopredict"},
		onenter = function(inst)
			inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("idle")
		end,
		
		events = {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        }
	},
}
    
return StateGraph("markmaniac", states, events, "idle", actionhandlers)